function [sys N]=buildModelFriction(TL)
%
% Creates the sys structure for the masses array
%
% TL = sample time of the fast model


N=4; % Number of masses
r=rand(N,1); % Reference vector
Ku=2;
ulim=0.7;        % Threshold that at most Ku inputs must exceed
deltaR=0.5;    % Max increment of position reference from current position

Tsim=500;
rr=r;
for i=1:Tsim
    rr=[rr r];
end

M=5*ones(N,1);% M(i)=mass of body #i
%M(2)=10;
%M(4)=4;
b=0.1*ones(N,1);        % b(i)=viscous friction of body #i
bp=.05*b;                   % viscous frinction between body #i ans #(i+1)
bm=.05*b;                   % viscous frinction between body #i ans #(i-1)
k=1*ones(N,1);        % k(i)=spring of body #i
kp=.1*ones(N,1);%.1      % kp(i)=spring between body #i and #(i+1)
km=.1*ones(N,1);%.1      % km(i)=spring between body #i and #(i-1)


y0=rand(N,1); % Initial positions (velocities are null)

%% Define full A,B model
%
% Definition of the global model. The discretized system is controlled by a
% lqr regulator which is mutilated in the non-diagonal elements...in order
% to have a completely decentralized Kgain (note: the degree of decentralization
% can be tuned according to the non-diagonal terms killed). This regulator
% have been tested in this example only
%


Ag=sparse(2*N,2*N);
Bg=zeros(2*N,N);
CgV=zeros(N,2*N);   % CgV Selection matrix for output velocities
CgP=CgV;            % CgP  Selection matrix for output positions
Dg=zeros(N,N);
x0=zeros(2*N,1);
for i=1:N,
    h=2*(i-1)+1;
    Ag(h,h+1)=1;             % velocity
    Ag(h+1,h+1)=-b(i)/M(i);  % friction
    Ag(h+1,h)=-k(i)/M(i);    % self-spring
    if i>1,
        Ag(h+1,h-1)=bm(i)/M(i);  % friction with #i-1
        Ag(h+1,h+1)=Ag(h+1,h)-bm(i)/M(i);  % friction with #i-1
    end
    if i<N,
        Ag(h+1,h+3)=bp(i)/M(i);  % friction with #i+1
        Ag(h+1,h+1)=Ag(h+1,h)-bp(i)/M(i);  % friction with #i+1
    end
    Bg(h+1,i)=1/M(i);
    CgP(i,h)=1;
    CgV(i,h+1)=1;
    x0(h)=y0(i);
end

sysgc=ss(full(Ag),Bg,CgV,zeros(N,N));

% Discretize in time
Ag=eye(2*N)+TL*Ag;
Bg=TL*Bg;

sysg=ss(Ag,Bg,CgV,zeros(N),TL); % Global system


%% Output and Input constraint definition

A=Ag;
B=Bg;
C=[];
for i=1:N
    C=[C;zeros(1,(i-1)*2) 1 0 zeros(1,(N-i)*2)];
end
D=zeros(N,N);
sys=ss(A,B,C,D,TL);
K = -dlqry(A,B,C,D,eye(N),eye(N));

K=K.*(CgP+CgV);

%save k.mat K
sys=ss((A+B*K),B,C,D,TL);
