% PLOT(WNmodel) plots structure of water network. It is useful to call PLOT 
% command as
%
%   MOD = PLOT(MOD)
%
% as plot automatically computes graph vertex position and stores it for
% later use. The computation can be time consuming for larger models (>50
% vertexes).
%
%
function obj = plot(obj)

orig_Lzw = obj.Lzw;
orig_Lzy = obj.Lzy;
orig_Luw = obj.Luw;
orig_Luy = obj.Luy;

% --- Forge L.. matrices ---------------------------
obj.Luy = zeros(size(obj.Luy));

M = obj.M;
n = obj.N;

for i = 1:n,            % submodels I/O dimensions
    nu(i) = size(M{i}.B,2);
    ny(i) = size(M{i}.C,1);    
end

valves_pumps = sum(obj.Luw,1)==2;
X = obj.Luw(:,valves_pumps);

clear C
for i=1:size(X,2),
  Xcell = mat2cell( X(:,i) , nu , 1);
  for j=1:size(Xcell,1),
    C(j) = any(any(Xcell{j,1}));
  end
  ind = find(C);
  if length(ind)~=2, error('???'); end
  obj.Luy(:,ind(1)) = min( obj.Luy(:,ind(1)) + X(:,i) , 1);
  obj.Luy(:,ind(2)) = min( obj.Luy(:,ind(2)) + X(:,i) , 1); 
end

obj.Luw = zeros(size(obj.Luw));
obj.Lzy = zeros(size(obj.Lzy));

% --- Call LSmodel plot --------------------------
obj = plot@LSmodel(obj);

title('\bfWater Distribution Network Scheme');

obj.Lzw = orig_Lzw;
obj.Lzy = orig_Lzy;
obj.Luw = orig_Luw;
obj.Luy = orig_Luy;

end