function [Nz,Nb] = CreateNzNb(nb,na,N)
%--------------------------------------------------------------------------
% Auxiliary function for FictiveRegressorData
% -------------------------------------------
% Creates data reorganization matrices allowing to vectorize structured
% regressor Z and RHS vector b:
%   vec(Z) = Nz*z, 
%   vec(Z) = Nb*z,
% where z is vector of unique parameters.
% Z and b have structure resulting from ARX identificaton -> two Hankel
% matrices.
%
% Inputs:
% nb ... input window length
% na ... output window length
% N  ... number of Z rows
%
%--------------------------------------------------------------------------

npar = N+nb-1 + N+na-1 + 1;  % number of parameters in unique vector z

Nz = [];
for i=1:nb,
  Nz = [ Nz ; zeros(N,i-1) , eye(N) zeros(N,npar-(i-1+N)) ];
end

offset = N+nb-1;
for i=1:na,
  Nz = [ Nz ; zeros(N,offset+i-1) , -eye(N) zeros(N,npar-(offset+i-1+N)) ];
end

Nb = [zeros(N,npar-N) , eye(N)];

