% -------------------------------------------------------------------------
% Models for demonstration of ARX models merging.
% ----------------------------------------
% Model from three different ID experiments (usually without proper
% excitation).
% -------------------------------------------------------------------------
function [id_mod,sys] = demo_models
randn('state',12345); rand('state',12345);


%--------------------------------------------------------------------------
% Paramters
%--------------------------------------------------------------------------
Ts = 0.2;
sigma_e = 0.2;      % noise filter variance
Ndata   = 300;      % number of samples for ID

% ----------------------------------

target_nb = 4; target_na = 4; % target model orders

na1=2;nb1=2;                  % model 1 orders
na2=3;nb2=3;                  % model 2 orders
na3=1;nb3=1;                  % model 3 orders

w1 = 2; ksi1 = 0.3;           % resonance frequencies
w2 = 5; ksi2 = 0.1;
sys = tf( w1^2,[1 2*w1*ksi1 w1^2] ) * tf( w2^2,[1 2*w2*ksi2 w2^2] );

% --- ID signals parameters --------
w_1  = w1*sqrt(1-ksi1^2); 
dw_1 = .3;
A_1  = 20;

w_2  = 4.8; 
dw_2 = .5;
A_2  = 20;

A_3  = 2;


% Discretization
sys = c2d(sys,Ts);
H   = tf(1,sys.den,Ts);

% --- ID ------------------------------------------------
ttt = Ts*(0:Ndata-1);

% --- Model 1 -------------------------------------------
ff1 = (w_1-dw_1)/(2*pi);
ff2 = (w_1+3*dw_1)/(2*pi);
u1  = A_1*chirp(ttt,ff1,max(ttt),ff2)';
y10 = lsim(sys,u1,ttt);
y1  = lsim(sys,u1,ttt) + lsim(H,sigma_e*randn(size(u1)),ttt);
data1 = iddata(y1,u1,Ts);
id_mod{1}  = arx(data1,[na1 nb1 1 ]);

% --- Model 2 -------------------------------------------
ff1 = (w_2-dw_2)/(2*pi);
ff2 = (w_2+3*dw_2)/(2*pi);
u2  = A_1*chirp(ttt,ff1,max(ttt),ff2)';
y20 = lsim(sys,u2,ttt);
y2  = lsim(sys,u2,ttt) + lsim(H,sigma_e*randn(size(u2)),ttt);
data2 = iddata(y2,u2,Ts);
id_mod{2}  = arx(data2,[na2 nb2 1 ]);

% --- Model 3 -------------------------------------------
z0 = [];
while length(z0)<=2*length(u1),
  z0 = [ z0 ones(1,round(3+17*rand)) zeros(1,round(5+45*rand)) -ones(1,round(3+17*rand)) zeros(1,round(5+45*rand))];
end
z0 = (z0(1:end-1) + z0(2:end))/2; % filter
z0 = A_3 * cumsum(z0(1:2*length(u1)))';
u3 = z0(1:length(u1));
y30 = lsim(sys,u3,ttt);
y3  = lsim(sys,u3,ttt) + lsim(H,sigma_e*randn(size(u3)),ttt);
data3 = iddata(y3,u3,Ts);
id_mod{3}  = arx(data3,[na3 nb3 1 ]);


