function [gridpnt0 setG const] = CreateGridpoints(tau,h,info)
% This function creates a column or matrix that contains the initial
% distribution of gridpoints. Different cases can be considered:
%   * Constant delay case
%   * Constant transmission interval case
%   * The case that both delays as transmission intervals are time varying.
%     These gridpoints span up space Theta (only small-delay case is
%     considered)
% -------------------------------------------------------------------------
% Input:
%   * taumin: minumim delay
%   * taumax: maximum delay
%   * hmin: minimum transmission interval
%   * hmax: maximum transmission interval
%   * info: if "info=true", inforhmaxon about the case under investigation 
%           is displayed on the command window
% -------------------------------------------------------------------------
% Output:
%   * gridpnt0: initial distribution of gridpoints
%   * setG:     string that contains informaxon about the case under
%               investigation, this will be used later on in the main GNB.m 
%               function file
% -------------------------------------------------------------------------
% Latest date of modification: 02-03-2011
% -------------------------------------------------------------------------
hmin = h(1);        hmax = h(2);
taumin = tau(1);    taumax = tau(2);

if taumin == taumax && hmin == hmax
    % both delay as transmission interval is constant
    gridpnt0 = [];
    setG = 'constht';
    const = [taumax hmax];
    if info == true
        disp('Case: both delay as transmission interval is constant')
    end
elseif taumin == taumax
    % constant delay
    gridpnt0 = [hmin;hmax];
    setG = 'varh';
    const = taumax;
    if info == true
        disp('Case: constant delay')
    end
elseif hmin == hmax
    % constant transmission interval
    gridpnt0 = [taumin;taumax];
    setG = 'vart';
    const = hmax;
    if info == true
        disp('Case: constant transmission interval')
    end
else
    if info == true
        disp('Case: both delays as transmission intervals are time varying')
    end
    setG = 'varht';
    const = [];
    if hmin > taumin && hmax > taumax
        gridpnt0 = [hmin   taumin
                    hmin   hmin
                    taumax taumax
                    hmax   taumax
                    hmax   taumin];
    elseif hmin == taumin && hmax > taumax
        gridpnt0 = [hmin   taumin
                    taumax taumax
                    hmax   taumax
                    hmax   taumin];
    elseif hmin == taumin && hmax == taumax
        gridpnt0 = [hmin taumin
                    hmax taumax
                    hmax taumin];
    elseif hmin > taumin && hmax == taumax
        gridpnt0 = [hmin taumin
                    hmin hmin
                    hmax taumax
                    hmax taumin];        
    elseif hmin >= taumax
        gridpnt0 = [hmin taumin
                    hmin taumax
                    hmax taumax
                    hmax taumin];
    end
end

end