function [S alfa] = exponent(J)
% EXPONENT      Determination of symbolic matrix exponential for real
%               and complex eigenvalues
% 
%   [S alfa] = exponent(J,eigSym,lambda) determines the symbolic matrix 
%   exponential of J*hk, where J is a Jordan block.  The outcome of 
%   this function is a cell S containing constant matrices and a cell
%   alfa containing uncertain time-varying terms as a function of h:
% 
%       expm(J*hk) = sum(alfa_i*S_i);
% 


syms hk
sizeJ = size(J,1);
alfa = cell(1,sizeJ);       %preallocate for better preformance
S = cell(1,sizeJ); %preallocate for better preformance

if size(J,1) > 1
    JLdiag = J(2,1);
else
    JLdiag =0;
end

if JLdiag == 0  %eigenvalues are real

    for i = 1:sizeJ
        alfa{i} = hk^(i-1)*exp(J(1,1)*hk);
        S{i} = 1/factorial(i-1)*(triu(ones(sizeJ),i-1)-triu(ones(sizeJ),i));
    end
    
else            %eigenvalues are complex
    sigma = J(1,1);
    omega = J(2,1);
    
    Ssinus = [0 -1; 1 0];
    for j = 1:(sizeJ/2-1)
        Ssinus = blkdiag(Ssinus,[0 -1; 1 0]);
    end

    for i = 1:sizeJ/2
        if i == 1
            alfa{i} = exp(sigma*hk)*cos(omega*hk);
            alfa{i+sizeJ/2} = exp(sigma*hk)*sin(omega*hk);
        else
            alfa{i} = hk^(i-1)*exp(simga*hk)*cos(omega*hk);
            alfa{i+sizeJ/2} = hk^(i-1)*exp(sigma*hk)*sin(omega*hk);
        end
        S{i} = 1/factorial(i-1)*(triu(ones(sizeJ),2*i-2)-triu(ones(sizeJ),2*i-1));
        S{i+sizeJ/2} = S{i}*Ssinus;
    end
end
