function stabilityBoundry = dncs_plotRobustness(OriginalNcs,res,ovrapx,arg1,arg2,arg3)
%% System details

hmin=OriginalNcs.h(1);
hmax=OriginalNcs.h(2);
taumin = OriginalNcs.tau(1);
taumax = OriginalNcs.tau(2);

hStep = (1/res)*(hmax-hmin);
tauStep = (1/res)*(taumax-taumin);

%% lower bound

ncs1 = OriginalNcs;
ncs1.h = [hmin hmax];
ncs1.tau = [taumin taumax];
stable = isNcsStable(ncs1,ovrapx,arg1,arg2,arg3);    %check stability

if stable == 0
   disp('System is unstable with given delay and transmission intervals!')
   disp('Cannot plot robustness bounds.')
   return
end

h = figure;     %create new figure
hold on;
xlabel('Transmission Interval (sec)')
ylabel('Delay (sec)')
title('Varying Delay vs Varying Transmission Interval Stability Region')
set(0,'CurrentFigure',h)
plot(hmin+(hmax-hmin)/2,taumin+(taumax-taumin)/2,'rx')                          %plot dummy point to generate legend
plot(hmin+(hmax-hmin)/2,taumin+(taumax-taumin)/2,'gs','MarkerFaceColor','g')    %plot dummy point to generate legend
plot(hmin+(hmax-hmin)/2,taumin+(taumax-taumin)/2,'bs','MarkerFaceColor','b')    %plot dummy point to generate legend
plot(0,0,'k.')                          %plot dummy point to hold axis
ncsData = patch([OriginalNcs.h(1) OriginalNcs.h(2) OriginalNcs.h(2) OriginalNcs.h(1)],[OriginalNcs.tau(1) OriginalNcs.tau(1) OriginalNcs.tau(2) OriginalNcs.tau(2)],'b');
legend ('Uncertain Region', 'Stable Region', 'User Defined Network' )
StabRegTotal = [];

%-----------------------------------------------------
%PART 1: find biggest box (search in both tau and h directions)

while stable == 1


    StabReg_Left(1,:) = [hmin taumax];          %define box corner
    StabReg_Left(2,:) = [hmin taumin];          %define box corner
    StabReg_Right(1,:) = [hmax taumin];         %define box corner
    StabReg_Right(2,:) = [hmax taumax];         %define box corner
    StabReg = [StabReg_Left; StabReg_Right];
    StabRegTotal = [StabRegTotal; StabReg];
    if exist('stablePoly','var') == 1    
        delete(stablePoly)
    end
    stablePoly = draw_area(h, StabRegTotal);
    %set(0,'CurrentFigure',h)
    %stableBox = patch(StabReg(:,1),StabReg(:,2),'g');         %plot stability box
    delete(ncsData)
    ncsData = patch([OriginalNcs.h(1) OriginalNcs.h(2) OriginalNcs.h(2) OriginalNcs.h(1)],[OriginalNcs.tau(1) OriginalNcs.tau(1) OriginalNcs.tau(2) OriginalNcs.tau(2)],'b');
    hmax = hmax + hStep;                %increase range (move increase box right edge)
    
    if taumax+tauStep <= hmin-hStep     % check if you are still under the taumax < hmin line
        taumax = taumax + tauStep;      % increase range (move increase box top edge) 
        
        if hmin -hStep > 0              % check for '0' boarder
            hmin = hmin - hStep;        % increase range (move increase left top edge) 
        else
            hmin = 0;
        end
       
    else
        taumax = hmin;                  % hold top and left edge below taumax < hmin line
    end
    
    if taumin-hStep > 0
        taumin = taumin - tauStep;        %increase range (move increase box bottom edge)
    else
        taumin = 0;
    end
    
    ncs1.h = [hmin hmax];
    ncs1.tau = [taumin taumax];
    stable = isNcsStable(ncs,ovrapx,arg1,arg2,arg3);    %check stability
    figure(h);
    drawnow;
end

hmin_box = hmin;        %save as starting point    
hmax_box = hmax;        %save as starting point    
taumin_box = taumin;    %save as starting point 
taumax_box = taumax;    %save as starting point 


%-----------------------------------------------------
%PART 2: search for stability range in (verical) tau direction

hmin = hmin_box;        %define starting point
hmax = hmax_box;        %define starting point
taumin = taumin_box;    %define starting point
taumax = taumax_box;    %define starting point

while hmin <= hmax

    if taumax <= hmin
        ncs1.h = [hmin hmax];
        ncs1.tau = [taumin taumax];  
        stable = isNcsStable(ncs1,ovrapx,arg1,arg2,arg3);    %check stability
    else
        stable = 0; 
    end
    
    if stable == 1
        StabReg_Left(1,:) = [hmin taumax];          %define box corner
        StabReg_Left(2,:) = [hmin taumin];          %define box corner
        StabReg_Right(1,:) = [hmax taumin];         %define box corner
        StabReg_Right(2,:) = [hmax taumax];         %define box corner
        StabReg = [StabReg_Left; StabReg_Right];
        StabRegTotal = [StabRegTotal; StabReg];
        delete(stablePoly)
        stablePoly = draw_area(h, StabRegTotal);
        %set(0,'CurrentFigure',h)
        %patch(StabReg(:,1),StabReg(:,2),'g')        %plot stability box
        delete(ncsData)
        ncsData = patch([OriginalNcs.h(1) OriginalNcs.h(2) OriginalNcs.h(2) OriginalNcs.h(1)],[OriginalNcs.tau(1) OriginalNcs.tau(1) OriginalNcs.tau(2) OriginalNcs.tau(2)],'b');

        
        taumax = taumax + tauStep;      %increase vertical range
        if taumin-tauStep>0             %check for '0' boarder
            taumin = taumin - tauStep;  %increase vertical range
        else
            taumin =0;
        end
        
    elseif stable == 0
        
        plot(hmin,taumin,'rx')
        plot(hmax,taumin,'rx')
        plot(hmax,taumax,'rx')
        plot(hmin,taumax,'rx')

        hmax = hmax - hStep;      %decrease range (narrow stability box)
        hmin = hmin + hStep;      %decrease range (narrow stability box)
    end
    figure(h);
    drawnow;
end
biggestTau = taumax;

%-----------------------------------------------------
%PART 3: search for stability range in (horizontal) 'h' direction

hmin = hmin_box;        %define starting point
hmax = hmax_box;        %define starting point
taumin = taumin_box;    %define starting point
taumax = taumax_box;    %define starting point

while taumin <= taumax

    if taumax <= hmin
        ncs1.h = [hmin hmax];
        ncs1.tau = [taumin taumax];  
        stable = isNcsStable(ncs1,ovrapx,arg1,arg2,arg3);    %check stability
    else
        stable = 0;
    end
    
    if stable == 1
        StabReg_Left(1,:) = [hmin taumax];          %define box corner
        StabReg_Left(2,:) = [hmin taumin];          %define box corner
        StabReg_Right(1,:) = [hmax taumin];         %define box corner
        StabReg_Right(2,:) = [hmax taumax];         %define box corner
        StabReg = [StabReg_Left; StabReg_Right];
        StabRegTotal = [StabRegTotal; StabReg];
        delete(stablePoly)
        stablePoly = draw_area(h, StabRegTotal);
        %set(0,'CurrentFigure',h) 
        %patch(StabReg(:,1),StabReg(:,2),'g')        %plot stability box
        delete(ncsData)
        ncsData = patch([OriginalNcs.h(1) OriginalNcs.h(2) OriginalNcs.h(2) OriginalNcs.h(1)],[OriginalNcs.tau(1) OriginalNcs.tau(1) OriginalNcs.tau(2) OriginalNcs.tau(2)],'b');
        
        hmax = hmax + hStep;      %increase horizontal range
        if hmin-hStep>0           %check for '0' boarder
            hmin = hmin - hStep;  %increase horizontal range
        else
            hmin =0;
        end
        
    elseif stable == 0
        
        plot(hmin,taumin,'rx')      %plot x
        plot(hmax,taumin,'rx')      %plot x
        plot(hmax,taumax,'rx')      %plot x
        plot(hmin,taumax,'rx')      %plot x

        taumax = taumax - tauStep;      %decrease range (narrow stability box)
        taumin = taumin + tauStep;      %decrease range (narrow stability box)
    end
    figure(h);
    drawnow;
end

delete(stablePoly)
[~,stabilityBoundry] = draw_area(h, StabRegTotal);
delete(ncsData)
patch([OriginalNcs.h(1) OriginalNcs.h(2) OriginalNcs.h(2) OriginalNcs.h(1)],[OriginalNcs.tau(1) OriginalNcs.tau(1) OriginalNcs.tau(2) OriginalNcs.tau(2)],'b');
        
% plot taumax = hmin line
plot([0 biggestTau],[0 biggestTau],'--')
disp('Tradeoff plot completed successfully !!!')


function [pic,q] = draw_area(h, p)

  % Sort vertices on x
  [ p(:,1), idx ] = sort(p(:,1));
  p(:,2) = p(idx,2);
  
  % Initialize result vectors
  q = zeros(0,2);
  
  % ...scan each quadrant seperately
  %
  %  Quadrants   1  |  2
  %             ----+----
  %              4  |  3
  %  
  for quadrant = 1:4
    
    % Are we going from right to left?
    switch quadrant
      
      case {1,4}        % Scan left to right
        x = p(:,1);
        y = p(:,2);
        
      case {2,3}        % Scan right to left
        x = flipud(p(:,1));
        y = flipud(p(:,2));
        
    end;
    
    % (Re)initialize result variables
    qt = zeros(0,2);
    ymm_cur = NaN;

    % Scan
    k = 1;
    while (k < length(x));

      % Select identical x's
      idx = find(x == x(k));
      
      % Determine new maximum (top) or minimum (bottom)
      switch quadrant
        
        case {1,2}    % Top
          ymm_nxt = max([ ymm_cur; y(idx) ]);
          
        case {3,4}    % Bottom
          ymm_nxt = min([ ymm_cur; y(idx) ]);
          
      end;
      
      % First point or, if there was a change, add coords
      if isnan(ymm_cur)
        qt = [ qt            ;
               x(k), ymm_nxt ];     % Add first vertex
      elseif (ymm_nxt ~= ymm_cur)
        qt = [ qt            ;
               x(k), ymm_cur ;      % Make a corner
               x(k), ymm_nxt ];     % Add vertex
      end;
      
      % Save current min/max
      ymm_cur = ymm_nxt;

      % Next set of identical x's
      k = k + length(idx);
    
    end;
    
    % Get orderning right (vertices should cw)
    switch quadrant
       
      case {1,3}
      	q = [q; qt];
       
      case {2,4}
        q = [q; flipud(qt)];
    end;
     
  end;
  
  % Plot results
  figure(h);
  pic = patch(q(:,1), q(:,2), 'g');
    
% end function
