function SKalmanFilter_UY(block)
% Level-2 M file S-Function for suboptimal
%quasi-steady-state Kalman Filter - Single output
  setup(block);
  
%endfunction

function setup(block)
  
  block.NumDialogPrms  = 13;
   %Parameter list
  %A,B,C            Plant State Space
  %Q, R             Noise covar matrices
  %nd               max network delay in the multiples of output sampling
  %                 time
  %ndopt            max dnetwork delay handled optimally 0<=ndopt<=nd
  %nmiss            buffer size for missed Y samples ndopt+nmiss<nd
  %nUmiss           buffer size for missed Uack samples nmiss<nd
  %figNo            Figure number for the barplot
  %pause            pause for vizualization
  %Tsmain, Tsmin    sampling periods main (control), minor (WSN)
  
  Tsmain=block.DialogPrm(12).Data;
  Tsmin=block.DialogPrm(13).Data;
  %-----------------------------------------------------------------------
  %% Register number of input and output ports
  block.NumInputPorts  = 5;
  block.NumOutputPorts = 2;%
  
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
 
  % measurements received via wireless network at high sampling rate
  block.InputPort(1).DatatypeID    = 0;
  block.InputPort(1).Complexity    = 0;
  block.InputPort(1).Dimensions    = 1;
  
  % measurement time stamps received via wireless network at high sampling rate
  block.InputPort(2).DatatypeID    = 0;
  block.InputPort(2).Complexity    = 0;
  block.InputPort(2).Dimensions    = 1;
  
% manipulated variables obtained from controller at low sampling rate 
    block.InputPort(3).DatatypeID    = 0;
    block.InputPort(3).Complexity    = 0;
    block.InputPort(3).Dimensions    = 1;

 % MV acknowledgements received via wireless network at high sampling rate
  block.InputPort(4).DatatypeID    = 0;
  block.InputPort(4).Complexity    = 0;
  block.InputPort(4).Dimensions    = 1;
  
  % MV acknowledgement time stmps received via wireless network at high sampling rate
  block.InputPort(5).DatatypeID    = 0;
  block.InputPort(5).Complexity    = 0;
  block.InputPort(5).Dimensions    = 1;
  
  %filtered CV output
    block.OutputPort(1).DatatypeID    = 0;
    block.OutputPort(1).Complexity    = 0;
    block.OutputPort(1).Dimensions    = 1; 
    
     %missed data (CV/MV acknowledgement)
    block.OutputPort(2).DatatypeID    = 8;
    block.OutputPort(2).Complexity    = 0;
    block.OutputPort(2).Dimensions    = 2; 
    
    
   %Delayed MV corr time stamps
  %-------------------------------end for tuning outputs  
    %Define sampling times
   block.InputPort(1).SampleTime  = [Tsmin 0];
   block.InputPort(2).SampleTime  = [Tsmin 0];
   block.InputPort(3).SampleTime  = [Tsmain   0];
   block.InputPort(4).SampleTime  = [Tsmin  0];
   block.InputPort(5).SampleTime  = [Tsmin  0];
   block.OutputPort(1).SampleTime = [Tsmain   0];
   block.OutputPort(2).SampleTime = [Tsmain   0];

%    Direct feedthroughs
  block.InputPort(1).DirectFeedthrough = true;
  block.InputPort(2).DirectFeedthrough = true; 
  block.InputPort(3).DirectFeedthrough = false;  
  block.InputPort(4).DirectFeedthrough = true;
  block.InputPort(5).DirectFeedthrough = true;  
     
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';

    block.SetAccelRunOnTLC(true);
  
  %% Register methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSampleTime',  @SetInpPortST);
  block.RegBlockMethod('SetOutputPortSampleTime', @SetOutPortST);
  block.RegBlockMethod('Update',                  @Update);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
  block.RegBlockMethod('SetInputPortDimensions', @SetInpPortDims);
  block.RegBlockMethod('SetInputPortDataType', @SetInpPortDataType);
  block.RegBlockMethod('SetOutputPortDataType', @SetOutPortDataType);
  block.RegBlockMethod('SetInputPortComplexSignal', @SetInpPortComplexSig);
  block.RegBlockMethod('SetOutputPortComplexSignal', @SetOutPortComplexSig);
  
%endfunction
function SetInpPortST(block, ~,~)
  ratio=(block.InputPort(1).SampleTime /block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(1).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction

function SetInpPortFrameData(block, idx, fd)
  
  block.InputPort(idx).SamplingMode = fd;
 for jj=1: block.NumOutputPorts
    block.OutputPort(jj).SamplingMode  = fd;
 end
 %endfunction

function SetInpPortDims(block, i, ~)
  
  block.InputPort(i).Dimensions = 1;       
    
%endfunction

function SetInpPortDataType(block, idx, dt)
  
  block.InputPort(idx).DataTypeID = dt;
%endfunction
  
function SetOutPortDataType(block, idx, dt)
  block.OutputPort(idx).DataTypeID  = dt;
  %block.InputPort(1).DataTypeID     = dt;
%endfunction  

function SetInpPortComplexSig(block, idx, c)
  
  block.InputPort(idx).Complexity = c;
 % block.OutputPort(1).Complexity  = c;
%endfunction 
  
function SetOutPortComplexSig(block, idx, c)

  block.OutputPort(idx).Complexity = c;
  %block.InputPort(1).Complexity    = c;
%endfunction 

function SetOutputPortSampleTime(s, i, time)

s.OutputPort(i).SampleTime = time;

function SetInputPortSamplingMode(s, ~, ~)

    for i=block.NumInputPorts
        s.InputPort(i).SamplingMode = 'Sample';
    end
    for jj=1: block.NumOutputPorts
     s.OutputPort(jj).SamplingMode = 'Sample';
    end
%endfunction

% Set output port sample time
function SetOutPortST(block, ~, ~)
  ratio=(block.InputPort(1).SampleTime \block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(6).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction
%.......................................................................
function DoPostPropSetup(block)

  %% Setup Dwork
  nd = block.DialogPrm(6).Data;
  ndopt=block.DialogPrm(7).Data;
  nmis=block.DialogPrm(8).Data;
  numis=block.DialogPrm(9).Data;
  nx0=size(block.DialogPrm(1).Data,1);
  
  [N,Nk,~,~,~,~]=DimInit(nx0,nd,ndopt,nmis);
  
  block.NumDworks = 23;
  block.Dwork(1).Name               = 'x';        %state
  block.Dwork(1).Dimensions         = nx0+nd;
  block.Dwork(1).DatatypeID         = 0;
  block.Dwork(1).Complexity         = 'Real';
  block.Dwork(1).UsedAsDiscState    = true;
  
  block.Dwork(2).Name               = 's';        %state of missing data 
  block.Dwork(2).Dimensions         = 1;
  block.Dwork(2).DatatypeID         = 5;
  block.Dwork(2).Complexity         = 'Real';
  block.Dwork(2).UsedAsDiscState    = true;
  
  block.Dwork(3).Name               = 'K';        %All Kalman Gains 
  block.Dwork(3).Dimensions         = Nk;
  block.Dwork(3).DatatypeID         = 0;
  block.Dwork(3).Complexity         = 'Real';
  block.Dwork(3).UsedAsDiscState    = false;

  block.Dwork(4).Name               = 'nc';        %Index into fields of K.G. 
  block.Dwork(4).Dimensions         = N+1;
  block.Dwork(4).DatatypeID         = 5;
  block.Dwork(4).Complexity         = 'Real';
  block.Dwork(4).UsedAsDiscState    = false;
  
  block.Dwork(5).Name               = 'misscnt';    %Count for data in missed smpl buffer
  block.Dwork(5).Dimensions         = 1;
  block.Dwork(5).DatatypeID         = 2;
  block.Dwork(5).Complexity         = 'Real';
  block.Dwork(5).UsedAsDiscState    = true;
  
  block.Dwork(6).Name               = 'DMiss';    %missed data buff
  block.Dwork(6).Dimensions         = max([nmis;1]);
  block.Dwork(6).DatatypeID         = 2;
  block.Dwork(6).Complexity         = 'Real';
  block.Dwork(6).UsedAsDiscState    = true;
  %----States related to U-buffering and replacement
  block.Dwork(7).Name               = 'UCorrGain';    %missed data buff
  block.Dwork(7).Dimensions         = max(numis,1)*(nx0+nd);
  block.Dwork(7).DatatypeID         = 0;
  block.Dwork(7).Complexity         = 'Real';
  block.Dwork(7).UsedAsDiscState    = true;
  
   block.Dwork(8).Name               = 'Umisscnt';    %Count for data in missed smpl buffer
  block.Dwork(8).Dimensions         = 1;
  block.Dwork(8).DatatypeID         = 2;
  block.Dwork(8).Complexity         = 'Real';
  block.Dwork(8).UsedAsDiscState    = true;
  
  block.Dwork(9).Name               = 'UDMiss';    %missed data buff
  block.Dwork(9).Dimensions         = max(numis,1);
  block.Dwork(9).DatatypeID         = 2;
  block.Dwork(9).Complexity         = 'Real';
  block.Dwork(9).UsedAsDiscState    = true;
  %
  %....................................................Y-data Sorter states
  block.Dwork(10).Name            = 'Ybuff';   %input buffer for incomming samples
  block.Dwork(10).Dimensions      = nd;
  block.Dwork(10).DatatypeID      = 0;
  block.Dwork(10).Complexity      = 0;
  block.Dwork(10).UsedAsDiscState = 1;
  
  block.Dwork(11).Name = 'Dbuff';              %input buffer for incomming samples'  time stamps
  block.Dwork(11).Dimensions      = nd;
  block.Dwork(11).DatatypeID      = 2;
  block.Dwork(11).Complexity      = 0;
  block.Dwork(11).UsedAsDiscState = 1;
  
  block.Dwork(12).Name = 'KFTime';              %Time of the last 'slow' sample hit
  block.Dwork(12).Dimensions      = 1;
  block.Dwork(12).DatatypeID      = 0;
  block.Dwork(12).Complexity      = 0;
  block.Dwork(12).UsedAsDiscState = 1;
  
  block.Dwork(13).Name  = 'RefDelStat';  %Delay sample status - without regard for possible limited buffering
  block.Dwork(13).Dimensions         = 1;
  block.Dwork(13).DatatypeID         = 5;
  block.Dwork(13).Complexity         = 'Real';
  block.Dwork(13).UsedAsDiscState    = true;
  
  block.Dwork(14).Name = 'YmissFlag';   %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(14).Dimensions      = 1;
  block.Dwork(14).DatatypeID      = 8;
  block.Dwork(14).Complexity      = 0;
  block.Dwork(14).UsedAsDiscState = 0;
 %.......................................................U-sorter-ack states 
   block.Dwork(15).Name = 'DUbuff';
  block.Dwork(15).Dimensions      = nd;
  block.Dwork(15).DatatypeID      = 0;
  block.Dwork(15).Complexity      = 0;
  block.Dwork(15).UsedAsDiscState = 1;
  
  block.Dwork(16).Name = 'Dbuff2';
  block.Dwork(16).Dimensions      = nd;
  block.Dwork(16).DatatypeID      = 2;
  block.Dwork(16).Complexity      = 0;
  block.Dwork(16).UsedAsDiscState = 1;
  
  block.Dwork(17).Name = 'UBuff';
  block.Dwork(17).Dimensions      = 2*nd+1;
  block.Dwork(17).DatatypeID      = 0;
  block.Dwork(17).Complexity      = 0;
  block.Dwork(17).UsedAsDiscState = 1;
  
   block.Dwork(18).Name = 'UAmissFlag';   %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(18).Dimensions      = 1;
  block.Dwork(18).DatatypeID      = 8;
  block.Dwork(18).Complexity      = 0;
  block.Dwork(18).UsedAsDiscState = 0;
  
  block.Dwork(19).Name  = 'RefDelStat2';  %Delay sample status - without regard for possible limited buffering
  block.Dwork(19).Dimensions         = 1;
  block.Dwork(19).DatatypeID         = 5;
  block.Dwork(19).Complexity         = 'Real';
  block.Dwork(19).UsedAsDiscState    = true;
  
  %-------------------------------------------------------Graph handles
  block.Dwork(20).Name = 'barplothandle';       %Graph handles
  block.Dwork(20).Dimensions      = 7;
  block.Dwork(20).DatatypeID      = 0;
  block.Dwork(20).Complexity      = 'Real';
  block.Dwork(20).UsedAsDiscState = true;
   %-----------------------------------
  block.Dwork(21).Name               = 'magic';        %a magic Integer to remember
  block.Dwork(21).Dimensions         = 1;
  block.Dwork(21).DatatypeID         = 5;
  block.Dwork(21).Complexity         = 'Real';
  block.Dwork(21).UsedAsDiscState    = false;
  %------------------------------------------
  block.Dwork(22).Name               = 'pow_of_2';        %a magic Integer to remember
  block.Dwork(22).Dimensions         = nd+1;
  block.Dwork(22).DatatypeID         = 5;
  block.Dwork(22).Complexity         = 'Real';
  block.Dwork(22).UsedAsDiscState    = false;
   %---------------------------------------------
  block.Dwork(23).Name = 'Statelock';              %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(23).Dimensions      = 1;
  block.Dwork(23).DatatypeID      = 8;
  block.Dwork(23).Complexity      = 0;
  block.Dwork(23).UsedAsDiscState = 1;
%endfunction

function InitConditions(block)

  A= block.DialogPrm(1).Data;
  C= block.DialogPrm(3).Data;
  Q = block.DialogPrm(4).Data;
  R = block.DialogPrm(5).Data;
  nd = block.DialogPrm(6).Data;
  ndopt=block.DialogPrm(7).Data;
  nmis=block.DialogPrm(8).Data;
  numis=block.DialogPrm(9).Data;
  figNo=block.DialogPrm(10).Data;
  nx0=size(A,1);
  
  %% Initialize Dwork
  block.Dwork(1).Data = zeros(nx0+nd,1);
  block.Dwork(2).Data = uint16(2^(ndopt+1)-1);
  [block.Dwork(3).Data,block.Dwork(4).Data]=...
      Get_Gain_set(A,C,Q,R,nd,ndopt,nmis);
   block.Dwork(5).Data = int8(0);
   block.Dwork(6).Data=zeros(max([nmis;1]),1,'int8');
   block.Dwork(7).Data=zeros(max(numis,1)*(nx0+nd),1);
   block.Dwork(8).Data = int8(0);
   block.Dwork(9).Data=zeros(max(numis,1),1,'int8');
   block.Dwork(13).Data = uint16(2^(nd+1)-1);
    block.Dwork(19).Data = uint16(2^(nd+1)-1);
    
     %!!!!!!!
     
    
   block.Dwork(10).Data=zeros(nd,1);
   block.Dwork(11).Data=-ones(nd,1,'int8');
   
   block.Dwork(15).Data=zeros(nd,1);
   block.Dwork(16).Data=-ones(nd,1,'int8');
   block.Dwork(17).Data=zeros(2*nd+1,1);
    
   if figNo>0
        InitGraph(block);
   end 
   c=zeros(nd+1,1);
   c(nd+1)=1;
   for i=1:nd
       c(nd+1-i)=2*c(nd+1-i+1);
   end
   block.Dwork(21).Data =uint16(sum(c(1:nd-ndopt))); 
   block.Dwork(22).Data=uint16(c);
   block.Dwork(23).Data=true;
%endfunction

function Output(block)
% Kalman data (filtering step implemented in the output Callback)
% Receives process measurements possibly delayed with time stamps as inputs
% Updates internal Kalman filter state (conditional means) based on these
% data). Sents filtered process output to the 

    nd = block.DialogPrm(6).Data;


 if block.OutputPort(1).IsSampleHit
     %%
    ndopt=block.DialogPrm(7).Data;
   
    X=block.Dwork(1).Data;
    nx0=length(X)-nd;
    C=block.DialogPrm(3).Data;
    if  block.Dwork(23).Data  %make sure that filtering and predicton steps 
                                %are run only once per sampling interval
        nmis=block.DialogPrm(8).Data;
        numis=block.DialogPrm(9).Data;
        figNo=block.DialogPrm(10).Data;
        pow2=block.Dwork(22).Data;
    
        M=zeros(nx0+nd,max(numis,1));
        M(:)= block.Dwork(7).Data;
%..........................................................................
%%
%sort data from wireless sources
        [Y, Del]=SYOutput(block);
        [DU,DelU]=SDUOutput(block);
    %tuning only
% .....Correcting for delayed U
    
        ndUm=block.Dwork(8).Data;
        DUmis=block.Dwork(9).Data(1:ndUm);
        DlstU=block.Dwork(19).Data;
        bU=Dec2Bin(DlstU-1,pow2);
%
        mU=length(DU);
        recov_indU=zeros(ndUm,1);rec_cntU=0;
        CurFlagU=0;
        for j=1:mU
            dj=DelU(j);
            mind=find(DUmis==dj, 1);
            idel=nd+1-dj;
            if ~bU(idel)
                DlstU=DlstU+pow2(idel);
                bU(idel)=1;
            end
            if ~isempty(mind)||dj==0
                if dj
                    rec_cntU=rec_cntU+1;
                    recov_indU(rec_cntU)=mind; 
                    duj=DU(j);
                    X=X+M(:,mind)*duj;
                else
                    CurFlagU=1;
                end
       
            end  %if ~isempty....
        end %for j
%reorganizing the list of missing Ydata
        [cntU,Dmis_updtU,reorg,missU]=...
            ReorgMissList(DUmis,recov_indU,rec_cntU,ndUm, numis,0,CurFlagU);
     %checking for missing data
        if cntU>0 && Dmis_updtU(cntU)>=nd
            missU=1;
%          disp('b')
        end
        if numis==0 && bU(nd+1)==0
            missU=1;
%          disp('c')
        end
    %Updating the state
        block.Dwork(9).Data(1:cntU)= Dmis_updtU;
        block.Dwork(8).Data=int8(cntU);
        block.Dwork(18).data=logical(missU);
        block.Dwork(19).Data=uint16(DlstU);
 %reorganize the M-matrix-create new column if actual MV not acknowledged;
 %put together columns for unused MV correction terms
        if CurFlagU
            indDU=1:cntU;
        else
            indDU=2:cntU;
        end
        M(:,indDU)=M(:,reorg);
%..........................................................................
%%
%.......Fusing Y data
        ndm=block.Dwork(5).Data;
        Dmis=block.Dwork(6).Data(1:ndm);
    
        m=length(Y);
%     b=GetBinR(Dmis,Del,block.Dwork(2).Data,nd,ndopt);
%     Dlst=Bin2Dec(b)+1;
        Dlst=block.Dwork(2).Data+block.Dwork(21).Data;
        Dlst=GetIndDlSt(Dmis,Del,Dlst,pow2);
%         disp(['***----------   ',int2str(Dlst)]) %%%
        b=Dec2Bin(Dlst-1,pow2);
    
        DlstX=block.Dwork(13).Data;
        bX=Dec2Bin(DlstX-1,pow2);
        K=block.Dwork(3).Data;
        nc=block.Dwork(4).Data;

        recov_ind=zeros(ndm,1);rec_cnt=0;
        LastFlag=b(nd+1-ndopt);
        for j=1:m
            dj=Del(j);
            edj=pow2(nd+1-dj);
            mind=find(Dmis==dj, 1);
            idel=nd+1-dj;
            if ~bX(idel)
                DlstX=DlstX+edj;
                bX(idel)=1;
            end
            if ~isempty(mind)||dj<=ndopt
                if dj
                    if dj>ndopt
                        rec_cnt=rec_cnt+1;
                        recov_ind(rec_cnt)=mind;
                    end
                    yj_hat=X(nx0+dj);
           
                else
                    yj_hat=C*X(1:nx0);
                end
                if dj==ndopt
                    LastFlag=1;
                end
                yj=Y(j);
                Ej=yj-yj_hat;
                a=~b;
                if a(idel)
                    kk=sum(a(1:idel));        
                    v1=nx0+nd-sum(b(1:nd))-1;
                    ij=nc(Dlst)+(kk-1)*v1;
                    indd=[1:nx0,nx0+find(a(nd:-1:1))];
                    if dj
                        indd=indd(indd~=nx0+dj);
                        X(nx0+dj)=yj;
                        ind2=ij+1:ij+v1;
                        M(indd,indDU)=M(indd,indDU)-K(ind2)*C*M(1:nx0,indDU);
                    else
                        ind2=ij+1:nc(Dlst+1);
                        M(indd,indDU)=M(indd,indDU)-K(ind2)*M(nx0+dj,indDU);
                    end
                    X(indd)=X(indd)+K(ind2)*Ej;
                  
                    b(idel)=1;
                    Dlst=Dlst+edj;
               
                end
            end  %if ~isempty....
        end %for j
%..........................................................................
%reorganizing the list of missing Ydata
        [cnt,Dmis_updt,~,missY]=ReorgMissList(Dmis,...
            recov_ind,rec_cnt,ndm, nmis,ndopt,LastFlag);
    %%
 %checking for missing data
        if cnt>0 && Dmis_updt(cnt)>=nd
            missY=1;
%          disp('b')
        end
        if nmis==0 && b(nd+1-ndopt)==0
            missY=1;
%          disp('c')
        end
  %  
        dlst0=1;
        for ii=nd-ndopt+1:nd+1
            dlst0=dlst0+b(ii)*pow2(ii);
        end
%Updating the state
        block.Dwork(6).Data(1:cnt)= Dmis_updt;
        block.Dwork(5).Data=int8(cnt);
        block.Dwork(2).Data=uint16(dlst0);
        block.Dwork(1).Data=X;
        block.Dwork(7).Data(:)=M;
        block.Dwork(13).Data=uint16(DlstX);
        block.Dwork(14).data=logical(missY);
        %update graphics
        if figNo>0
            UpdateGraph(block)
        end
    end
% Updating output ports
    block.OutputPort(1).Data=C*X(1:nx0);
    block.OutputPort(2).Data=[ block.Dwork(14).data; block.Dwork(18).Data];
    block.Dwork(23).Data=false;
%---------------------------------------------------------------------------    
    
 end        %is sample hit
%endfunction
%%

function Update(block)
%State update - Kalman filter prediction step
 
nd = block.DialogPrm(6).Data;
 
if block.InputPort(3).IsSampleHit
     block.Dwork(17).Data(2:2*nd+1)=block.Dwork(17).Data(1:2*nd);
     block.Dwork(17).Data(1)=block.InputPort(3).Data;
    ndopt=block.DialogPrm(7).Data;
    numis=block.DialogPrm(9).Data;
    X=block.Dwork(1).Data;
    nx0=length(X)-nd;
    U=block.InputPort(3).Data;
    A=block.DialogPrm(1).Data; 
    B=block.DialogPrm(2).Data;
    C=block.DialogPrm(3).Data;

    Y=C*X(1:nx0);
    X(1:nx0)=A*X(1:nx0);
    X(1:nx0)=X(1:nx0)+B*U;
    X(nx0+2:nx0+nd)=X(nx0+1:nx0+nd-1);
    X(nx0+1)=Y;
    block.Dwork(1).Data=X;
    
    pow2=block.Dwork(22).Data;
    mrd=2*pow2(1);
    if ndopt<nd
        auxx=pow2(nd-ndopt);
    else
        auxx=mrd;
    end
    block.Dwork(2).Data=mod(2*block.Dwork(2).Data-2,auxx)+1;
    block.Dwork(13).Data=mod(2*block.Dwork(13).Data-2,mrd)+1;
    block.Dwork(19).Data=mod(2*block.Dwork(19).Data-2,mrd)+1;
    
    M=zeros(nx0+nd,max(numis,1));
    M(:)= block.Dwork(7).Data;
    cntU=block.Dwork(8).Data;
    LastFlg=block.Dwork(9).Data(1)~=0;
    if ~LastFlg
        M(1:nx0,1)=B;
        M(nx0+1:nx0+nd,1)=zeros(nd,1);
        indDU=2:cntU;
    else
        indDU=1:cntU;
    end
    M(nx0+2:nx0+nd,indDU)=M(nx0+1:nx0+nd-1,indDU);
    M(1:nx0+1,indDU)=[A;C]*M(1:nx0,indDU);

    block.Dwork(7).Data(:)=M;
    [block.Dwork(5).Data,block.Dwork(6).Data]=...
        UpdtMissList(block.Dwork(5).Data,block.Dwork(6).Data,nd);
    [block.Dwork(8).Data,block.Dwork(9).Data]=...
        UpdtMissList(block.Dwork(8).Data,block.Dwork(9).Data,nd);
    
    block.Dwork(23).Data=true; %unlock state 
else
    %sort out incomming data from WSN
    %  ... sensor channel
    ts=block.InputPort(2).Data;
    if ts>-1
        dts=(block.Dwork(12).Data-ts)/block.OutputPort(1).SampleTime(1);
        aux=int8(dts)+1;
        if aux<=nd
            y=block.InputPort(1).Data;
            block.Dwork(10).Data(2:nd)=block.Dwork(10).Data(1:nd-1);
            block.Dwork(10).Data(1)=y;
            block.Dwork(11).Data(2:nd)=block.Dwork(11).Data(1:nd-1);
            block.Dwork(11).Data(1)=aux;
        end  
    end
    %... actuator channel
    ts=block.InputPort(5).Data;
    if ts>-1
        dts=(block.Dwork(12).Data-ts)/block.OutputPort(1).SampleTime(1);
        tau=block.InputPort(4).Data;
        dtaus=dts+tau;
        aux=int8(dts)+1;
        if aux<=nd  && tau <=nd
            uref=block.Dwork(17).Data(dts+1);
            ureal=block.Dwork(17).Data(dtaus+1);
            block.Dwork(15).Data(2:nd)=block.Dwork(15).Data(1:nd-1);
            block.Dwork(15).Data(1)=ureal-uref;
            block.Dwork(16).Data(2:nd)=block.Dwork(16).Data(1:nd-1);
            block.Dwork(16).Data(1)=int8(dts)+1;
        end
    end
end
%%
%endfunction
%>>>>>>>>>>>>>>>>>Local helper functions serving the callbacks
%Y-Sorter output function
function [Ybuff, Dbuff]=SYOutput(block)

        nd=block.DialogPrm(6).Data; %max delay
        TimeS=block.CurrentTime;
        block.Dwork(12).Data=TimeS;
        y=block.InputPort(1).Data;
        ts=block.InputPort(2).Data;
        if ts>-1
            dts=(TimeS-ts)/block.OutputPort(1).SampleTime(1);
            if dts<=nd
                Ybuff=[y;block.Dwork(10).Data];
                Dbuff=[int8(dts);block.Dwork(11).Data];
            else
                Ybuff=block.Dwork(10).Data;
                Dbuff=block.Dwork(11).Data;
            end
        else
            Ybuff=block.Dwork(10).Data;
            Dbuff=block.Dwork(11).Data;
        end
       
        block.Dwork(10).Data=zeros(nd,1);
        block.Dwork(11).Data=-ones(nd,1,'int8');
        iy=find(Dbuff>-1);
        [Dbuff,ind]=sort(Dbuff(iy),1,'ascend');
        Ybuff=Ybuff(iy(ind));
%endfunction
%%
%U-Sorter output function
function [Ybuff,Dbuff]=SDUOutput(block)
     
nd=block.DialogPrm(6).Data; %max delay
        TimeS=block.CurrentTime;
%         block.Dwork(12).Data=TimeS;
        tau=block.InputPort(4).Data;
        ts=block.InputPort(5).Data;
        if ts>-1
            dts=(TimeS-ts)/block.OutputPort(1).SampleTime(1);
            if dts>0 && dts<=nd && tau <=nd
                dtaus=dts+tau;
                uref=block.Dwork(17).Data(dts);
                ureal=block.Dwork(17).Data(dtaus);
                Dbuff=[int8(dts);block.Dwork(16).Data];
                Ybuff=[ureal-uref;block.Dwork(15).Data];
            else
                Ybuff=block.Dwork(15).Data;
                Dbuff=block.Dwork(16).Data;
            end
        else
            Ybuff=block.Dwork(15).Data;
            Dbuff=block.Dwork(16).Data;
        end
        iy=find(Dbuff>-1);
        [Dbuff,ind]=sort(Dbuff(iy),1,'descend');
        Ybuff=Ybuff(iy(ind));
        block.Dwork(15).Data=zeros(nd,1);
        block.Dwork(16).Data=-ones(nd,1,'int8');
   %%

function [N,nk,a,nc,v1,v2]=DimInit(nx0,nd,ndopt,nmis)
    %initialize dimensions of parameter vectors and provide some
    %intermediate results
N=2^(nd+1);
a=zeros(N,nd+1);
for i=2:N
    a(i,:)=incr_bin_arr(a(i-1,:));
end
%
bb=~a;
 v0=sum(bb,2);v1=nx0+nd-sum(a(:,1:nd),2)-1;
 nki=v0.*v1+bb(:,end);
 v2=sum(bb(:,1:nd-ndopt),2)<=nmis;
 if ~isempty(v2)
     nki=nki.*v2;
 end
 nc=uint16(cumsum([0;nki]));
 nk=nc(end);
 %endfunction
 
 function a1=incr_bin_arr(a)
% increment a binary array
n=length(a);
a1=a;
if a1(n)
    if n>1
        a1(1:n-1)=incr_bin_arr(a1(1:n-1));
    else
        disp('Warning: binary length overflow')
    end
end
a1(n)=~a1(n);


 function [KK,nc]=Get_Gain_set(A0,C0,Q0,R0,nd,ndopt,nmis)
% Compute the vector of Kalman gains at start-up
nx0=size(A0,2);

[N,~,a,nc,v1,v2]=DimInit(nx0,nd,ndopt,nmis);
%
A=[A0,zeros(nx0,nd);[C0;zeros(nd-1,nx0)],[zeros(1,nd);eye(nd-1),zeros(nd-1,1)]];
Q=[Q0,zeros(nx0,nd);zeros(1,nx0),R0,zeros(1,nd-1);zeros(nd-1,nx0+nd)];
C=[A(nx0+1:nx0+nd,:);zeros(1,nx0+nd-1),1];
R=[R0,zeros(1,nd);zeros(nd,nd+1)];
Cci=zeros(1,nx0+nd);Cci(1,nx0+nd)=1;
CovMstrct=cell(N);
CovMstrct{1}=dare(A',Cci',Q,0);
KK=zeros(nc(end),1);
 
 for i=1:N-1
     MM=CovMstrct{i};
     aa=a(i,:); 
     na=~aa;
     ind0=[1:nx0,nx0+find(fliplr(na(1:nd)))];
     m=sum(na);
     next=zeros(1,m);
     jj=0;
     for j=1:nd+1
        if na(j)
            jj=jj+1;
            dj=(nd+1-j);
            next(jj)=i+2^dj;
            dj=dj+1;
            Cj=C(dj,:);
            Rj=R(dj,dj);
            Hj=MM*Cj';
            Wj=Cj*Hj+Rj;
            Kj=Hj/Wj;
            ij=nc(i)+(jj-1)*v1(i);
            if dj-1>0
               indd=ind0(ind0~=nx0+dj-1); 
               ind_2=ij+1:ij+v1(i);
            else
                indd=ind0;
                ind_2=ij+1:nc(i+1);
            end
            if v2(i)
                KK(ind_2)=Kj(indd);
            end
            if isempty(CovMstrct{next(jj)})
                Mnext=MM-Kj*Hj';
                CovMstrct{next(jj)}=Mnext;
            end
        end
     end
 end
%% 
 function Dlst=GetIndDlSt(DelMis,Del,Dlst,c)

    nd=length(c)-1;
for i=1:length(DelMis)
    dmi=DelMis(i);
    for j=1:length(Del)
        dj=Del(j);
        if dmi==dj
            Dlst=Dlst-c(nd+1-dj);
        end
    end
end

function a=Dec2Bin(d,c)
%Decimal numbwer to binary representation
n=length(c);
a=zeros(1,n);

if d>=c(1)*2 
    error('overflow')
end
for i=1:n
    if d>=c(i)
        a(i)=1;
        d=d-c(i);
    end
    
end
%%
function [cnt, Dmis_updt, reorg,miss]=...
        ReorgMissList(Dmis,recov_ind,rec_cnt,ndm,nmis,ndopt,LastFlag)
    
 recov_ind=recov_ind(1:rec_cnt);
    ndm_updt=ndm-rec_cnt;
    if ~LastFlag
        ndm_updt=min(ndm_updt+1,nmis);
    end
    Dmis_updt=zeros(ndm_updt,1,'int8');
     reorg=zeros(ndm_updt,1);
    cnt=0;
    if ~LastFlag &&nmis>0
        Dmis_updt(1)=ndopt;
        cnt=1;
    end
    miss=0;
    for i=1:ndm
        
        pom=find(i==recov_ind,1);
        if isempty(pom)
            di=Dmis(i);
          if cnt<nmis
              cnt=cnt+1;
              Dmis_updt(cnt)=di;
              reorg(cnt)=i;
          else
              miss=1;
          end
        end
    end
     Dmis_updt= Dmis_updt(1:cnt);
     if ~LastFlag
        reorg=reorg(2:cnt);
     else
         reorg=(1:cnt);
     end
     %%
     
     function [cnt,Buff]=UpdtMissList(cnt,Buff,nd)
if cnt>0
    Buff(1:cnt)=Buff(1:cnt)+1;
    if Buff(cnt)>nd
        cnt=cnt-1;
    end
end
%%

%>>>Graph-related functions
function InitGraph(block)

  %% Initialize Dwork
  nd = block.DialogPrm(6).Data;
  ndopt= block.DialogPrm(7).Data;
  fig= block.DialogPrm(10).Data;
 figure(fig);clf
%set(fg,'color',[0.,0.,0])


set(gcf,'windowstyle','normal')
set(gcf,'position',[50,500,550,300])
set(gcf,'Color',[0,0,0])
[hh,hh0,tt,tt2]=set_plott(fig, [.075,.65,.775,.15],...
    'Received CV Data Status',nd,ndopt, 1);

[hh1,hh2,~,tt3]=set_plott(fig, [.075,.2,.775,.15],...
    'Received MV ackn Status ',nd,0, 0);

  block.Dwork(20).Data=[hh;hh0;tt;tt2; hh1;hh2;tt3];
%endfunction
%%
 function [hh,hh0,tt,tt2]=set_plott(fgr, position,titl,nd,ndopt, timopt)
        
 figure(fgr)
 axes('position',position);
 hh0=bar(nd:-1:0,zeros(1,nd+1)); hold on
 hh=bar(nd:-1:0,ones(1,nd+1)); hold off
 ylim([0,1]);xlim([-1,nd+1]);
% title('Received CV Data Status','fontsize',14,'fontweight','demi')
 set(gca,'visible','off')

set(hh,'facecolor',[0.1686 0.5059 0.3373])
set(hh,'edgecolor',[0,1,0])
set(hh,'showbaseline','off')
set(hh,'LineWidth',3)

set(hh0,'facecolor',[1,1,0])
set(hh0,'edgecolor',[0,1,0])
set(hh0,'showbaseline','off')
set(hh0,'LineWidth',3)

for i=0:nd
    aux=text(i,-.25,int2str(i),'fontsize',14,...
        'horizontalalignment','center');
    if i==ndopt
        set(aux,'color',[1,1,0],'fontweight','demi');
    else
        set(aux,'color',[.8,.8,.8],'fontweight','normal');
    end
end
text(-.5,1.4,titl,'color',[.8,.8,.8],...
    'fontsize',14);
tt=[];
if timopt
    tt=text(nd+.5,1.8,'00:00:00');
    set(tt,'fontsize',14);
    set(tt,'color','yellow');
end
tt2=text(nd+.5,-.25,'smpl loss');
set(tt2,'fontsize',14);
set(tt2,'fontweight','demi');
set(tt2,'color',[0,.5,0]);
%endfunction

function UpdateGraph(block)
nd = block.DialogPrm(6).Data;
pau = block.DialogPrm(11).Data;
%fig=block.DialogPrm(1).Data;
s= block.Dwork(13).Data-1;
% disp([block.CurrentTime,s])
a=Dec2Bin(s,block.Dwork(22).Data);

ndm=block.Dwork(5).Data;
Dmis=block.Dwork(6).Data(1:ndm);
b=zeros(1,nd+1);
for i=1:ndm
    dm=Dmis(i);
    b(nd+1-dm)=1;
end
Hndls=block.Dwork(20).Data;
%figure(fig)
updtax(Hndls(1),Hndls(2),Hndls(3),Hndls(4),a,b,block.CurrentTime,...
    block.Dwork(14).Data)
s= block.Dwork(19).Data-1;
a=Dec2Bin(s,block.Dwork(22).Data);
ndm=block.Dwork(8).Data;
Dmis=block.Dwork(9).Data(1:ndm);
b=zeros(1,nd+1);
for i=1:ndm
    dm=Dmis(i);
    b(nd+1-dm)=1;
end
updtax(Hndls(5),Hndls(6),[],Hndls(7),a,b,block.CurrentTime,...
    block.Dwork(18).Data)
pause(max(1e-3,pau))
% endfunction

function updtax(hh,hh1,tt,tt1,a,b,stim,MissFlag)

set(hh,'ydata',a);
set(hh1,'ydata',b);
if ~isempty(tt)
    hr=floor(stim/3600);
    stim=stim-hr*3600;
    mn=floor(stim/60);
    sc=floor(stim-60*mn);
    hr=int2str(hr); if length(hr)<2, hr=['0',hr];  end
    mn=int2str(mn); if length(mn)<2, mn=['0',mn];  end
    sc=int2str(sc); if length(sc)<2, sc=['0',sc];  end
    set(tt,'String',[hr,':',mn,':',sc])
end
if MissFlag
    set(tt1,'BackgroundColor',[.66,0,0])
    set(tt1,'color',[0.8 0.8 0]);
else
   set(tt1,'BackgroundColor','none')
   set(tt1,'color',[0,.5,0]);
end