function SKalmanFilter_Yswitch(block)
% Level-2 M file S-Function for suboptimal
%quasi-steady-state Kalman Filter - Single output
  setup(block);
  
%endfunction

function setup(block)
  
  block.NumDialogPrms  = 12;
  %Parameter list
  %A,B,C            Plant State Space
  %Q, R             Noise covar matrices
  %nd               max network delay in the multiples of output sampling
  %                 time
  %ndopt            max dnetwork delay handled optimally 0<=ndopt<=nd
  %nmiss            buffer size for missed samples ndopt+nmiss<Lnd
  %figNo            Figure number for the barplot (0 - no plot)
  %pause            pause for vizualization
  %Ts, Tsmin        Controller sampling time/network sampling time
  
  %-----------------------------------------------------------------------
  Ts = block.DialogPrm(11).Data;
  Tsmin = block.DialogPrm(12).Data;
  %% Register number of input port and output port
  block.NumInputPorts  = 3;
  block.NumOutputPorts = 2;

  %% Setup functional port properties
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
% measurement data received via wireless network at high sampling rate
  block.InputPort(1).DatatypeID    = 0;
  block.InputPort(1).Complexity    = 0;
  block.InputPort(1).Dimensions    = 1;
  
  % measurement timestamps  received via wireless network at high sampling rate
  block.InputPort(2).DatatypeID    = 0;
  block.InputPort(2).Complexity    = 0;
  block.InputPort(2).Dimensions    = 1;
  
% manipulated variables obtained from controller at low sampling rate 
   block.InputPort(3).DatatypeID    = 0;
   block.InputPort(3).Complexity    = 0;
   block.InputPort(3).Dimensions    = 1;

  %filtered CV
  block.OutputPort(1).DatatypeID    = 0;
  block.OutputPort(1).Complexity    = 0;
  block.OutputPort(1).Dimensions    = 1; 
  
 %missed data (CV/MV acknowledgement)
   block.OutputPort(2).DatatypeID    = 8;
   block.OutputPort(2).Complexity    = 0;
   block.OutputPort(2).Dimensions    = 1; 
  
%Define sampling times
   block.InputPort(1).SampleTime  = [Tsmin 0];
   block.InputPort(2).SampleTime  = [Tsmin 0];
   block.InputPort(3).SampleTime  = [Ts   0];
   block.OutputPort(1).SampleTime = [Ts   0];
   block.OutputPort(2).SampleTime = [Ts   0];

  
  block.InputPort(1).DirectFeedthrough = true;
  block.InputPort(2).DirectFeedthrough = true;
  block.InputPort(3).DirectFeedthrough = false;
  
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';

  block.SetAccelRunOnTLC(true);
  
  %% Reg methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSampleTime',  @SetInpPortST);
  block.RegBlockMethod('SetOutputPortSampleTime', @SetOutPortST);
  block.RegBlockMethod('Update',                  @Update);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
  block.RegBlockMethod('SetInputPortDimensions', @SetInpPortDims);
  block.RegBlockMethod('SetInputPortDataType', @SetInpPortDataType);
  block.RegBlockMethod('SetOutputPortDataType', @SetOutPortDataType);
  block.RegBlockMethod('SetInputPortComplexSignal', @SetInpPortComplexSig);
  block.RegBlockMethod('SetOutputPortComplexSignal', @SetOutPortComplexSig);
  
%endfunction
  % Set input port sample time
function SetInpPortST(block, ~,~)
  ratio=(block.InputPort(1).SampleTime \block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(1).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction

function SetInpPortFrameData(block, idx, fd)
  
  block.InputPort(idx).SamplingMode = fd;
 
    block.OutputPort(1).SamplingMode  = fd;
    block.OutputPort(2).SamplingMode  = fd;
 %endfunction

function SetInpPortDims(block, idx, dim)
  
  block.InputPort(idx).Dimensions = dim;       
%   block.InputPort(2).Dimensions = 1;       
%endfunction

function SetInpPortDataType(block, idx, dt)
  
  block.InputPort(idx).DataTypeID = dt;
%endfunction
  
function SetOutPortDataType(block, idx, dt)
  block.OutputPort(idx).DataTypeID  = dt;
  %block.InputPort(1).DataTypeID     = dt;
%endfunction  

function SetInpPortComplexSig(block, idx, c)
  
  block.InputPort(idx).Complexity = c;
 % block.OutputPort(1).Complexity  = c;
%endfunction 
  
function SetOutPortComplexSig(block, idx, c)

  block.OutputPort(idx).Complexity = c;
  %block.InputPort(1).Complexity    = c;
%endfunction 

function SetOutputPortSampleTime(s, i, time)

s.OutputPort(i).SampleTime = time;

function SetInputPortSamplingMode(s, ~, ~)

    for i=1:3
        s.InputPort(i).SamplingMode = 'Sample';
    end
     s.OutputPort(1).SamplingMode = 'Sample';
%endfunction

% Set output port sample time
function SetOutPortST(block, ~, ~)
  ratio=(block.InputPort(1).SampleTime /block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(6).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction
%------------------------------------------------------------------------
%------------------------------------------------------------------------
function DoPostPropSetup(block)

  %% Setup Dwork
  nd = block.DialogPrm(6).Data;
  ndopt=block.DialogPrm(7).Data;
  nmis=block.DialogPrm(8).Data;
  nx0=size(block.DialogPrm(1).Data,1);
  
  [N,Nk,~,~,~,~]=DimInit(nx0,nd,ndopt,nmis);
  
  block.NumDworks = 15;
  block.Dwork(1).Name               = 'x';        %state
  block.Dwork(1).Dimensions         = nx0+nd;
  block.Dwork(1).DatatypeID         = 0;
  block.Dwork(1).Complexity         = 'Real';
  block.Dwork(1).UsedAsDiscState    = true;
  
  block.Dwork(2).Name               = 's';        %state of missing data 
  block.Dwork(2).Dimensions         = 1;
  block.Dwork(2).DatatypeID         = 5;
  block.Dwork(2).Complexity         = 'Real';
  block.Dwork(2).UsedAsDiscState    = true;
  
  block.Dwork(3).Name               = 'K';        %All Kalman Gains 
  block.Dwork(3).Dimensions         = Nk;
  block.Dwork(3).DatatypeID         = 0;
  block.Dwork(3).Complexity         = 'Real';
  block.Dwork(3).UsedAsDiscState    = false;

  block.Dwork(4).Name               = 'nc';        %Index into fields of K.G. 
  block.Dwork(4).Dimensions         = N+1;
  block.Dwork(4).DatatypeID         = 5;
  block.Dwork(4).Complexity         = 'Real';
  block.Dwork(4).UsedAsDiscState    = false;
  
  block.Dwork(5).Name               = 'misscnt';    %Count for data in missed smpl buffer
  block.Dwork(5).Dimensions         = 1;
  block.Dwork(5).DatatypeID         = 2;
  block.Dwork(5).Complexity         = 'Real';
  block.Dwork(5).UsedAsDiscState    = true;
  
  block.Dwork(6).Name               = 'DMiss';    %missed data buff
  block.Dwork(6).Dimensions         = max([nmis;1]);
  block.Dwork(6).DatatypeID         = 2;
  block.Dwork(6).Complexity         = 'Real';
  block.Dwork(6).UsedAsDiscState    = true;
  
  %....................................................Y-data Sorter states
  block.Dwork(7).Name            = 'Ybuff';   %input buffer for incomming samples
  block.Dwork(7).Dimensions      = nd;
  block.Dwork(7).DatatypeID      = 0;
  block.Dwork(7).Complexity      = 0;
  block.Dwork(7).UsedAsDiscState = 1;
  
  block.Dwork(8).Name = 'Dbuff';              %input buffer for incomming samples'  time stamps
  block.Dwork(8).Dimensions      = nd;
  block.Dwork(8).DatatypeID      = 2;
  block.Dwork(8).Complexity      = 0;
  block.Dwork(8).UsedAsDiscState = 1;
  
  block.Dwork(9).Name = 'KFTime';              %Time of the last 'slow' sample hit
  block.Dwork(9).Dimensions      = 1;
  block.Dwork(9).DatatypeID      = 0;
  block.Dwork(9).Complexity      = 0;
  block.Dwork(9).UsedAsDiscState = 1;
  
  block.Dwork(10).Name  = 'RefDelStat';        %Delay sample status - without regard for possible limited buffering
  block.Dwork(10).Dimensions         = 1;
  block.Dwork(10).DatatypeID         = 5;
  block.Dwork(10).Complexity         = 'Real';
  block.Dwork(10).UsedAsDiscState    = true;
  
  block.Dwork(11).Name = 'YmissFlag';              %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(11).Dimensions      = 1;
  block.Dwork(11).DatatypeID      = 8;
  block.Dwork(11).Complexity      = 0;
  block.Dwork(11).UsedAsDiscState = 0;
  %--------
  block.Dwork(12).Name = 'barplothandle';       %Graph handles
  block.Dwork(12).Dimensions      = 4;
  block.Dwork(12).DatatypeID      = 0;
  block.Dwork(12).Complexity      = 'Real';
  block.Dwork(12).UsedAsDiscState = true;
  %-----------------------------------
  block.Dwork(13).Name               = 'magic';        %a magic Integer to remember
  block.Dwork(13).Dimensions         = 1;
  block.Dwork(13).DatatypeID         = 5;
  block.Dwork(13).Complexity         = 'Real';
  block.Dwork(13).UsedAsDiscState    = false;
  %------------------------------------------
  block.Dwork(14).Name               = 'pow_of_2';        %a magic Integer to remember
  block.Dwork(14).Dimensions         = nd+1;
  block.Dwork(14).DatatypeID         = 5;
  block.Dwork(14).Complexity         = 'Real';
  block.Dwork(14).UsedAsDiscState    = false;
  %---------------------------------------------
  block.Dwork(15).Name = 'Statelock';              %Flag - true if data were lost in the sampling interval (either not arrived or due to insufficient buffer lenghts
  block.Dwork(15).Dimensions      = 1;
  block.Dwork(15).DatatypeID      = 8;
  block.Dwork(15).Complexity      = 0;
  block.Dwork(15).UsedAsDiscState = 1;
%endfunction

function InitConditions(block)

  A= block.DialogPrm(1).Data;
  C= block.DialogPrm(3).Data;
  Q = block.DialogPrm(4).Data;
  R = block.DialogPrm(5).Data;
  nd = block.DialogPrm(6).Data;
  ndopt=block.DialogPrm(7).Data;
  nmis=block.DialogPrm(8).Data;
  figNo=block.DialogPrm(9).Data;
  nx0=size(A,1);
  
  %% Initialize Dwork
  block.Dwork(1).Data = zeros(nx0+nd,1);
  block.Dwork(2).Data = uint16(2^(ndopt+1)-1);
  block.Dwork(10).Data = uint16(2^(ndopt+1)-1);
  [block.Dwork(3).Data,block.Dwork(4).Data]=...
      Get_Gain_set(A,C,Q,R,nd,ndopt,nmis);
   block.Dwork(5).Data = int8(0);
   block.Dwork(6).Data=zeros(max([nmis;1]),1,'int8');
   
   block.Dwork(7).Data=zeros(nd,1);
   block.Dwork(8).Data=-ones(nd,1,'int8');
   if figNo>0
        InitGraph(block);
   end
   
   c=zeros(nd+1,1);
   c(nd+1)=1;
   for i=1:nd
       c(nd+1-i)=2*c(nd+1-i+1);
   end

   block.Dwork(13).Data =uint16(sum(c(1:nd-ndopt))); 
   block.Dwork(14).Data=uint16(c);
   block.Dwork(15).Data=true;
%endfunction

function Output(block)
% Kalman data (filtering step implemented in the output Callback)
% Receives process measurements possibly delayed with time stamps as inputs
% Updates internal Kalman filter state (conditional means) based on these
% data). Sents filtered process output to the 

%%
 if block.OutputPort(1).IsSampleHit 
     nd = block.DialogPrm(6).Data;
     X=block.Dwork(1).Data;
    nx0=length(X)-nd;
    C=block.DialogPrm(3).Data;
    if  block.Dwork(15).Data  %make sure that filtering and predicton steps 
                                %are run only once per sampling interval 
        ndopt=block.DialogPrm(7).Data;
        nmis=block.DialogPrm(8).Data;
        [block, Y, Del]=SOutput(block);
        ndm=block.Dwork(5).Data;
        Dmis=block.Dwork(6).Data(1:ndm);
    
        figNo=block.DialogPrm(9).Data;
        pow2=block.Dwork(14).Data;
        m=length(Del);

        Dlst=block.Dwork(2).Data+block.Dwork(13).Data;
        Dlst=GetIndDlSt(Dmis,Del,Dlst,pow2);
        b=Dec2Bin(Dlst-1,pow2);
%
        DlstX=block.Dwork(10).Data;
        aX = Dec2Bin(DlstX-1,pow2);
        K=block.Dwork(3).Data;
        nc=block.Dwork(4).Data;
    %%
        recov_ind=zeros(ndm,1);rec_cnt=0;
        LastFlag=b(nd+1-ndopt);
        for j=1:m
            dj=Del(j);
%         edj=2^dj;
            idel=nd+1-dj;
            edj=pow2(idel);
            mind=find(Dmis==dj, 1);
        
            if ~aX(idel)
                DlstX=DlstX+edj;
                aX(idel)=1;
            end
            if ~isempty(mind)||dj<=ndopt
                if dj
                    if dj>ndopt
                        rec_cnt=rec_cnt+1;
                        recov_ind(rec_cnt)=mind;
                    end
                    yj_hat=X(nx0+dj);
                else
                    yj_hat=C*X(1:nx0);
                end
                if dj==ndopt
                    LastFlag=1;
                end
                yj=Y(j);
                Ej=yj-yj_hat;
                a=~b;
                if a(idel)
                    kk=sum(a(1:idel));        
                    v1=nx0+nd-sum(b(1:nd))-1;
                    ij=nc(Dlst)+(kk-1)*v1;
                    indd=[1:nx0,nx0+find(a(nd:-1:1))];
                    if dj
                        indd=indd(indd~=nx0+dj);
                        X(nx0+dj)=yj;
                        ind2=ij+1:ij+v1;
                    else
                        ind2=ij+1:nc(Dlst+1);
                    end
                    X(indd)=X(indd)+K(ind2)*Ej;
                    b(idel)=1;
                    Dlst=Dlst+edj;

                end
            end  %if ~isempty....
        end %for j
    %%
    %reorganizing the list of missing delays
        recov_ind=recov_ind(1:rec_cnt);
        ndm_updt=ndm-rec_cnt;
        if ~LastFlag
            ndm_updt=min(ndm_updt+1,nmis);
        end
        Dmis_updt=zeros(ndm_updt,1,'int8');
        cnt=0;
        if ~LastFlag
            Dmis_updt(1)=ndopt;
            cnt=min(nmis,1);
        end
        missY=0;
%      Dmis0=Dmis;  %debug only
        for i=1:ndm
            pom=find(i==recov_ind,1);
            if isempty(pom)
                di=Dmis(i);
                if cnt<nmis
                    cnt=cnt+1;
                    Dmis_updt(cnt)=di;
                else
                    missY=1;
%               disp('a')
                end
            end
        end
    %%
    %checking for missing data
        if cnt>0 && Dmis_updt(cnt)>=nd
            missY=1;
%          disp('b')
        end
        if nmis==0 && b(nd+1-ndopt)==0
            missY=1;
%          disp('c')
        end
    %%
        dlst0=1;
        for ii=nd-ndopt+1:nd+1
            dlst0=dlst0+b(ii)*pow2(ii);
        end
%Updating the state
        block.Dwork(6).Data(1:cnt)= Dmis_updt(1:cnt);
        block.Dwork(5).Data=int8(cnt);
        block.Dwork(2).Data=uint16(dlst0);
        block.Dwork(10).Data=uint16(DlstX);
        block.Dwork(1).Data=X;
        block.Dwork(11).data=logical(missY);
         %update graphics
        if figNo>0
            UpdateGraph(block)
        end
    end
% Updating output ports
    block.OutputPort(1).Data=C*X(1:nx0);
    block.OutputPort(2).Data=block.Dwork(11).data;
    block.Dwork(15).Data=false; %lock the state
%    
 end
%endfunction

function Update(block)
%State update - Kalman filter prediction step
 
nd = block.DialogPrm(6).Data;
if block.OutputPort(1).IsSampleHit
    ndopt=block.DialogPrm(7).Data;
    X=block.Dwork(1).Data;
    nx0=length(X)-nd;
    U=block.InputPort(3).Data;
    A=block.DialogPrm(1).Data; 
    B=block.DialogPrm(2).Data;
    C=block.DialogPrm(3).Data;

    Y=C*X(1:nx0);
    X(1:nx0)=A*X(1:nx0);
    X(1:nx0)=X(1:nx0)+B*U;
    X(nx0+2:nx0+nd)=X(nx0+1:nx0+nd-1);
    X(nx0+1)=Y;
    block.Dwork(1).Data=X;
    pow2=block.Dwork(14).Data;
    mrd=2*pow2(1);
    if ndopt<nd
        auxx=pow2(nd-ndopt);
    else
        auxx=mrd;
    end
    block.Dwork(2).Data=mod(2*block.Dwork(2).Data-2,auxx)+1;
    block.Dwork(10).Data=mod(2*block.Dwork(10).Data-2,mrd)+1;

    cnt=block.Dwork(5).Data;
    if cnt>0
        block.Dwork(6).Data(1:cnt)=block.Dwork(6).Data(1:cnt)+1;
        if block.Dwork(6).Data(cnt)>nd
            block.Dwork(5).Data=cnt-1;
        end
    end
      block.Dwork(15).Data=true; %unlock state 
else
    %sort out incomming data from WSN
    ts=block.InputPort(2).Data(1);
    if ts>-1
        dts=(block.Dwork(9).Data-ts)/block.OutputPort(1).SampleTime(1);
        aux=int8(dts)+1;
        if aux<=nd
            y=block.InputPort(1).Data(1);
            block.Dwork(7).Data(2:nd)=block.Dwork(7).Data(1:nd-1);
            block.Dwork(7).Data(1)=y;
            block.Dwork(8).Data(2:nd)=block.Dwork(8).Data(1:nd-1);
            block.Dwork(8).Data(1)=aux;
        end
        
    end
end

%endfunction
%>>>>>>>>>>>>>>>>>Local helper functions serving the callbacks
%Y-Sorter output function
function [block, Ybuff, Dbuff]=SOutput(block)

        nd=block.DialogPrm(6).Data; %max delay
        TimeS=block.CurrentTime;
        block.Dwork(9).Data=TimeS;
        y=block.InputPort(1).Data;
        ts=block.InputPort(2).Data;
        if ts>-1
            dts=(TimeS-ts)/block.OutputPort(1).SampleTime(1);
            if dts<=nd
                Ybuff=[y;block.Dwork(7).Data];
                Dbuff=[int8(dts);block.Dwork(8).Data];
            else
                Ybuff=block.Dwork(7).Data;
                Dbuff=block.Dwork(8).Data;
            end
        else
            Ybuff=block.Dwork(7).Data;
            Dbuff=block.Dwork(8).Data;
        end
       
        block.Dwork(7).Data=zeros(nd,1);
        block.Dwork(8).Data=-ones(nd,1,'int8');
        iy=find(Dbuff>-1);
        [Dbuff,ind]=sort(Dbuff(iy),1,'ascend');
        Ybuff=Ybuff(iy(ind));
        
%endfunction

function [N,nk,a,nc,v1,v2]=DimInit(nx0,nd,ndopt,nmis)
    %initialize dimensions of parameter vectors and provide some
    %intermediate results
N=2^(nd+1);
a=zeros(N,nd+1);
for i=2:N
    a(i,:)=incr_bin_arr(a(i-1,:));
end
%
bb=~a;
 v0=sum(bb,2);v1=nx0+nd-sum(a(:,1:nd),2)-1;
 nki=v0.*v1+bb(:,end);
 v2=sum(bb(:,1:nd-ndopt),2)<=nmis;
 if ~isempty(v2)
     nki=nki.*v2;
 end
 nc=uint16(cumsum([0;nki]));
 nk=nc(end);
 
 function [KK,nc]=Get_Gain_set(A0,C0,Q0,R0,nd,ndopt,nmis)
% Compute the vector of Kalman gains
nx0=size(A0,2);

[N,~,a,nc,v1,v2]=DimInit(nx0,nd,ndopt,nmis);
%
A=[A0,zeros(nx0,nd);[C0;zeros(nd-1,nx0)],[zeros(1,nd);eye(nd-1),zeros(nd-1,1)]];
Q=[Q0,zeros(nx0,nd);zeros(1,nx0),R0,zeros(1,nd-1);zeros(nd-1,nx0+nd)];
C=[A(nx0+1:nx0+nd,:);zeros(1,nx0+nd-1),1];
R=[R0,zeros(1,nd);zeros(nd,nd+1)];
Cci=zeros(1,nx0+nd);Cci(1,nx0+nd)=1;
CovMstrct=cell(N);
CovMstrct{1}=dare(A',Cci',Q,0);

KK=zeros(nc(end),1);
 
 for i=1:N-1
     MM=CovMstrct{i};
     aa=a(i,:); 
     na=~aa;
     ind0=[1:nx0,nx0+find(fliplr(na(1:nd)))];
     m=sum(na);
     next=zeros(1,m);
     jj=0;
     for j=1:nd+1
        if na(j)
            jj=jj+1;
            dj=(nd+1-j);
            next(jj)=i+2^dj;
            dj=dj+1;
            Cj=C(dj,:);
            Rj=R(dj,dj);
            Hj=MM*Cj';
            Wj=Cj*Hj+Rj;
            Kj=Hj/Wj;
            ij=nc(i)+(jj-1)*v1(i);
            if dj-1>0
               indd=ind0(ind0~=nx0+dj-1); 
               ind_2=ij+1:ij+v1(i);
            else
                indd=ind0;
                ind_2=ij+1:nc(i+1);
            end
            if v2(i)
                KK(ind_2)=Kj(indd);
            end
            if isempty(CovMstrct{next(jj)})
                Mnext=MM-Kj*Hj';
                CovMstrct{next(jj)}=Mnext;
            end
        end
     end
 end
 
function Dlst=GetIndDlSt(DelMis,Del,Dlst,c)

    nd=length(c)-1;
for i=1:length(DelMis)
    dmi=DelMis(i);
    for j=1:length(Del)
        dj=Del(j);
        if dmi==dj
            Dlst=Dlst-c(nd+1-dj);
        end
    end
end
function a1=incr_bin_arr(a)
% increment a binary array
n=length(a);
a1=a;
if a1(n)
    if n>1
        a1(1:n-1)=incr_bin_arr(a1(1:n-1));
    else
        disp('Warning: binary length overflow')
    end
end
a1(n)=~a1(n);
        
function a=Dec2Bin(d,c)
%Decimal numbwer to binary representation
n=length(c);
a=zeros(1,n);

if d>=c(1)*2 
    error('overflow')
end
for i=1:n
    if d>=c(i)
        a(i)=1;
        d=d-c(i);
    end
    
end
%>>>Graph-related functions
function InitGraph(block)

  %% Initialize Dwork
  nd = block.DialogPrm(6).Data;
  ndopt= block.DialogPrm(7).Data;
  fig= block.DialogPrm(9).Data;
 figure(fig);
%set(fg,'color',[0.,0.,0])
hh0=bar(nd:-1:0,zeros(1,nd+1)); hold on
hh=bar(nd:-1:0,zeros(1,nd+1)); hold off
ylim([0,1]);xlim([-1,nd+1]);
% title('Received CV Data Status','fontsize',14,'fontweight','demi')
set(gca,'visible','off')

set(hh,'facecolor',[0.1686 0.5059 0.3373])
set(hh,'edgecolor',[0,1,0])
set(hh,'showbaseline','off')
set(hh,'LineWidth',3)

set(hh0,'facecolor',[1,1,0])
set(hh0,'edgecolor',[0,1,0])
set(hh0,'showbaseline','off')
set(hh0,'LineWidth',3)

for i=0:nd
    aux=text(i,-.25,int2str(i),'fontsize',14,...
        'horizontalalignment','center');
    if i==ndopt
        set(aux,'color',[1,1,0],'fontweight','demi');
    else
        set(aux,'color',[.8,.8,.8],'fontweight','normal');
    end
end
%     if ndopt<nd
%         line([ndopt,ndopt],[0,1],'linewidth',3,'linestyle','--','color',[.75,.75,.75])
%     end
set(gca,'color',[0.,0.,0.])
set(gcf,'windowstyle','normal')
set(gcf,'position',[50,725,550,125])
set(gca,'position',[.075,.25,.775,.4])
set(gcf,'Color',[0,0,0])
ttx=text(-.5,1.4,'Received CV Data Status');
set(ttx,'fontsize',14);

set(ttx,'color',[.8,.8,.8]);

tt=text(nd+.5,1.6,'00:00:00');
set(tt,'fontsize',14);

set(tt,'color','cyan');

tt2=text(nd+.5,-.25,'smpl loss');
set(tt2,'fontsize',14);
set(tt2,'fontweight','demi');
 set(tt2,'color',[0,1,0]);
%set(tt2,'color',[.25,.25,.25]);
set(tt2,'backgroundcolor','none');
  block.Dwork(12).Data=[hh;hh0;tt;tt2];
%endfunction

function UpdateGraph(block)
nd = block.DialogPrm(6).Data;
pau = block.DialogPrm(10).Data;
%fig=block.DialogPrm(1).Data;
s= block.Dwork(10).Data-1;
a=Dec2Bin(s,block.Dwork(14).Data);%-----------------------------------

ndm=block.Dwork(5).Data;
Dmis=block.Dwork(6).Data(1:ndm);
b=zeros(1,nd+1);
for i=1:ndm
    dm=Dmis(i);
    b(nd+1-dm)=1;
end

%figure(fig)
set(block.Dwork(12).Data(1),'ydata',a);
set(block.Dwork(12).Data(2),'ydata',b);
stim=block.CurrentTime;
hr=floor(stim/3600);
stim=stim-hr*3600;
mn=floor(stim/60);
sc=floor(stim-60*mn);
hr=int2str(hr); if length(hr)<2, hr=['0',hr];  end
mn=int2str(mn); if length(mn)<2, mn=['0',mn];  end
sc=int2str(sc); if length(sc)<2, sc=['0',sc];  end
set(block.Dwork(12).Data(3),'String',[hr,':',mn,':',sc])
if block.Dwork(11).Data
    set(block.Dwork(12).Data(4),'BackgroundColor',[.66,0,0])
    set(block.Dwork(12).Data(4),'color',[0.8 0.8 0]);
else
   set(block.Dwork(12).Data(4),'BackgroundColor','none')
   set(block.Dwork(12).Data(4),'color',[0,.5,0]);
end
pause(max(1e-3,pau))
