function WNReceiver(block)
% Level-2 M file S-Function for multirate wireless demo:
%
% Catching samples with time stamp arriving from Wireless network with a
% fast sampling rate and sending the latest sample to the actuator at the
% baseline control sampling rate

  setup(block);
%endfunction

function setup(block)
  
  block.NumDialogPrms  = 3;
  Tsmain=block.DialogPrm(2).Data;
  Tsmin=block.DialogPrm(3).Data;
%% Register number of input port and output port
  block.NumInputPorts  = 2;
  block.NumOutputPorts = 2;

  %% Setup functional port properties
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;

  block.InputPort(1).DatatypeID    = 0;
  block.InputPort(1).Complexity    = 0;
  block.InputPort(1).Dimensions    = 1;
  
   block.InputPort(2).DatatypeID    = 0;
  block.InputPort(2).Complexity    = 0;
  block.InputPort(2).Dimensions    = 1;

    %selected output output value
    block.OutputPort(1).Dimensions        = 1;
    block.OutputPort(1).DatatypeID        = 0;
   
    block.OutputPort(2).Dimensions        = 1;
    block.OutputPort(2).DatatypeID        = 0;
%   block.OutputPort(1).Complexity        = 0;

   block.InputPort(1).SampleTime  = [Tsmin 0];
   block.InputPort(2).SampleTime  = [Tsmin 0];
   block.OutputPort(1).SampleTime = [Tsmain 0];
   block.OutputPort(2).SampleTime = [Tsmain 0];
  
  block.InputPort(1).DirectFeedthrough = true;
  block.InputPort(2).DirectFeedthrough = true;
  %% Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';
  block.SetAccelRunOnTLC(true);
  
  %% Reg methods

  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('SetInputPortSampleTime',  @SetInpPortST);
  block.RegBlockMethod('SetOutputPortSampleTime', @SetOutPortST);
  block.RegBlockMethod('Update',                  @Update);  
  block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);
  block.RegBlockMethod('SetInputPortDimensions', @SetInpPortDims);
  %block.RegBlockMethod('SetOutputPortDimensions', @SetOutPortDims);
  block.RegBlockMethod('SetInputPortDataType', @SetInpPortDataType);
  block.RegBlockMethod('SetOutputPortDataType', @SetOutPortDataType);
  block.RegBlockMethod('SetInputPortComplexSignal', @SetInpPortComplexSig);
  block.RegBlockMethod('SetOutputPortComplexSignal', @SetOutPortComplexSig);
  
%endfunction
% Set input port sample time
function SetInpPortST(block, ~,~)
  ratio=(block.InputPort(1).SampleTime \block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(1).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end

%endfunction


function SetInpPortFrameData(block, idx, fd)
  
  block.InputPort(idx).SamplingMode = fd;
  for i=1:2
    block.OutputPort(i).SamplingMode  = fd;
  end
 
%endfunction

function SetInpPortDims(block, i, dim)
  
  block.InputPort(i).Dimensions = dim;       
  

%endfunction

function SetInpPortDataType(block, idx, dt)
  
  block.InputPort(idx).DataTypeID = dt;

%endfunction
  
function SetOutPortDataType(block, idx, dt)

  block.OutputPort(idx).DataTypeID  = dt;
%   block.InputPort(1).DataTypeID     = dt;

%endfunction  

function SetInpPortComplexSig(block, idx, c)
  
  block.InputPort(idx).Complexity = c;
%   block.OutputPort(1).Complexity  = c;

%endfunction 
  
function SetOutPortComplexSig(block, idx, c)

  block.OutputPort(idx).Complexity = c;
%   block.InputPort(1).Complexity    = c;

%endfunction 


function SetOutputPortSampleTime(s, i, time)

s.OutputPort(i).SampleTime = time;
% s.OutputPort(2).SampleTime = time;
% 
% 
function SetInputPortSamplingMode(s, i, ~)

s.InputPort(i).SamplingMode = 'Sample';
s.OutputPort(1).SamplingMode = 'Sample';
s.OutputPort(2).SamplingMode = 'Sample';


% Set output port sample time
function SetOutPortST(block, ~, ~)
  ratio=(block.InputPort(1).SampleTime /block.OutputPort(1).SampleTime);
  if ratio~=round(ratio), 
      error('Output Sampling pedriod not an integer multiple of the input one')
  end
  if ratio<block.DialogPrm(1).Data
      warning(1,'Possibly insufficient number of time slots in the basic sampling interval')
  end
%endfunction
% Do post-propagation process
function DoPostPropSetup(block)
%% Setup DWork

  nd= block.DialogPrm(1).Data;
  
  block.NumDworks = 5;
  
  block.Dwork(1).Name = 'Ybuff';
  block.Dwork(1).Dimensions      = nd;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 0;
  block.Dwork(1).UsedAsDiscState = 1;
  
  block.Dwork(2).Name = 'Dbuff';
  block.Dwork(2).Dimensions      = nd;
  block.Dwork(2).DatatypeID      = 2;
  block.Dwork(2).Complexity      = 0;
  block.Dwork(2).UsedAsDiscState = 1;
  
  block.Dwork(3).Name = 'Time';
  block.Dwork(3).Dimensions      = 1;
  block.Dwork(3).DatatypeID      = 0;
  block.Dwork(3).Complexity      = 0;
  block.Dwork(3).UsedAsDiscState = 1;

  block.Dwork(4).Name               = 'lastY';     %latest MV received
  block.Dwork(4).Dimensions         = 1;
  block.Dwork(4).DatatypeID         = 0;
  block.Dwork(4).Complexity         = 'Real';
  block.Dwork(4).UsedAsDiscState    = true;
  
  block.Dwork(5).Name               = 'lastD';     %delay of the latest MV received
  block.Dwork(5).Dimensions         = 1;
  block.Dwork(5).DatatypeID         = 0;
  block.Dwork(5).Complexity         = 'Real';
  block.Dwork(5).UsedAsDiscState    = true;
  

%endfunction


function InitConditions(block)

  %% Initialize Dwork
  nd = block.DialogPrm(1).Data;
  
  block.Dwork(1).Data=zeros(nd,1);
  block.Dwork(2).Data=-ones(nd,1,'int8');
  block.Dwork(4).Data = 0;
  block.Dwork(5).Data = 0;
 
  
%endfunction

function Output(block)

   if block.OutputPort(1).IsSampleHit 
        nd=block.DialogPrm(1).Data; %max delay
        TimeS=block.CurrentTime;
        block.Dwork(3).Data=TimeS;
        y=block.InputPort(1).Data;
        ts=block.InputPort(2).Data;
        if ts>-1
            Y=[y;block.Dwork(1).Data];
            dts=(TimeS-ts)/block.OutputPort(1).SampleTime(1);
            Del=[int8(dts);block.Dwork(2).Data];
        else
            Y=[block.Dwork(1).Data;0];
            Del=[block.Dwork(2).Data;-1];
        end
        
        block.Dwork(1).Data=zeros(nd,1);
        block.Dwork(2).Data=-ones(nd,1,'int8');
        
        iy=find(Del>-1 &Del<=nd);
% m=length(iy);
    Y=[Y(iy);block.Dwork(4).Data];
    Del=[Del(iy);block.Dwork(5).Data];
    [dd,i]=min(Del);
    yy=Y(i);
    block.OutputPort(1).Data=yy;
    block.OutputPort(2).Data=double(dd);
    block.Dwork(4).Data=yy;
    block.Dwork(5).Data=block.OutputPort(2).Data;
   end
  
%endfunction

function Update(block)

nd=block.DialogPrm(1).Data;
if ~block.OutputPort(1).IsSampleHit
    ts=block.InputPort(2).Data;
    if ts>-1
        y=block.InputPort(1).Data;
        block.Dwork(1).Data(2:nd)=block.Dwork(1).Data(1:nd-1);
        block.Dwork(1).Data(1)=y;
        block.Dwork(2).Data(2:nd)=block.Dwork(2).Data(1:nd-1);
        dts=(block.Dwork(3).Data-ts)/block.OutputPort(1).SampleTime(1);
        block.Dwork(2).Data(1)=int8(dts)+1;
        
    end
    
else
    block.Dwork(5).Data=block.Dwork(5).Data+1;
end

%endfunction
