function msfcn_burst(block)
% Gilber-Elliot (G-E) burst model for Multi-hop network model
% Simulink block sfunction

setup(block);
  
%endfunction


%% Block Setup
function setup(block)
  
  block.NumDialogPrms  = 2;
  
  % --- Block parameters from dialog ----------------------
  p_error    = block.DialogPrm(1).Data;
  P_recovery = block.DialogPrm(2).Data;
  
  % Register number of input and output ports
  block.NumInputPorts  = 0;
  block.NumOutputPorts = 1;

  % Setup functional port properties to dynamically inherited.
  block.SetPreCompInpPortInfoToDynamic;
  block.SetPreCompOutPortInfoToDynamic;
   
  block.OutputPort(1).Complexity   = 'Real';
  block.OutputPort(1).DataTypeId   = 0;
  block.OutputPort(1).SamplingMode = 'Sample';    
  block.OutputPort(1).Dimensions   = 1;
      
  % Set block sample time to inherited
  block.SampleTimes = [-1 0];
  
  % Set the block simStateCompliance to default (i.e., same as a built-in block)
  block.SimStateCompliance = 'DefaultSimState';

  % Register methods
  block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
  block.RegBlockMethod('InitializeConditions',    @InitConditions);  
  block.RegBlockMethod('Outputs',                 @Output);  
  block.RegBlockMethod('Update',                  @Update);  
  
%endfunction


%% Block Post Setup
function DoPostPropSetup(block)

  % --- Setup Dwork ---------------------------------------
  block.NumDworks = 1;
  block.Dwork(1).Name = 'state'; 
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 0;
  block.Dwork(1).Complexity      = 'Real';
  block.Dwork(1).UsedAsDiscState = false;  

%endfunction


%% Block Init Conditions 
function InitConditions(block)

  % --- Initialize Dwork ----------------------------------
  block.Dwork(1).Data = 0;
  
%endfunction


%% Block Output Function
function Output(block)

  block.OutputPort(1).Data = block.Dwork(1).Data;
  
%endfunction


%% Block Update
function Update(block)
  
  % --- Block parameters from dialog ----------------------
  p_error    = block.DialogPrm(1).Data;
  p_recovery = block.DialogPrm(2).Data;
  
  % --- Load data from DWork ------------------------------
  state = block.Dwork(1).Data;  
  
  % --- State switching -----------------------------------
  if ~state
    if rand<=p_error,
      state = 1;
    end
  else
    if rand<=p_recovery,
      state = 0;
    end    
  end
  
  % --- Save data to DWork --------------------------------
  block.Dwork(1).Data = state;
    
%endfunction
